#ifdef HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */
#include        "sadie.h"

/*-Copyright Information------------------------------------------------------*/
/* Copyright (c) 1988 by the University of Arizona Digital Image Analysis Lab */
/*----------------------------------------------------------------------------*/
/*-General Information--------------------------------------------------------*/
/*                                                                            */
/*   This function combines several images to form a single multi-band        */
/*   image (for disk or tape I/O). The input images may themselves be         */
/*   multi-band, in which case the bands are concatenated sequentially.       */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/*-Interface Information------------------------------------------------------*/
void BSQTOBIL (
IMAGE **in,     /*  I   Pointer to an array[nimg] containing pointers         */
                /*      to the input images.                                  */
uint32_t nimg,  /*  I   Number of input images.                               */
IMAGE **out     /*  O   Address of a pointer to the output image.             */
/*----------------------------------------------------------------------------*/
) { register uint32_t i, j, k, l, m, nbnd, nlin, npix;
    char   msg[SLEN];
    double pinc, psum;

    if (TIMES) TIMING(T_BSQTOBIL);
    if (NAMES) {
        MESSAGE('I',"");
        MESSAGE('I',"BSQTOBIL");
    }

    /* check input */
    if (nimg < 1) {
        MESSAGE('E'," Number of input images must be greater than zero.");
        goto the_end;
    }
    for (i=0; i<nimg; i++) {
        if (!CHECKIMG(in[i])) {
            MESSAGE('E'," Can't identify image(s).");
            goto the_end;
        }
    }
    for (nlin=in[0]->nlin,i=1; i<nimg; i++) {
        if (nlin != in[i]->nlin) {
            MESSAGE('E'," Number of lines must be identical in all images.");
            goto the_end;
        }
    }
    for (npix=in[0]->npix,i=1; i<nimg; i++) {
        if (npix != in[i]->npix) {
            MESSAGE('E'," Number of pixels/line must be identical in all images.");
            goto the_end;
        }
    }

    if (NAMES) {
        MESSAGE('I',"");
        sprintf(msg," Number of input images:   %lu",nimg);
        MESSAGE('I',msg);
        for (i=0; i<nimg; i++) {
            sprintf(msg," Input image:              %s",in[i]->text);
            MESSAGE('I',msg);
        }
        MESSAGE('I'," ...............");
    }

    /* create image of appropriate size */
    for (nbnd=0,i=0; i<nimg; nbnd += in[i++]->nbnd) {
        continue;
    }
    if (!CHECKIMG(*out)) GETMEM(nbnd,nlin,npix,out);
    if (!*out) goto the_end;

    /* initialize progress indicator */
    if (LINES  &&  !PROGRESS(psum=0.0)) goto the_end;
    pinc = 1.0/(double)nbnd/(double)nlin;

    /* combine images */
    for (m=0,i=0; i<nimg; i++) {
        for (j=0; j<in[i]->nbnd; j++,m++) {
            for (k=0; k<nlin; k++) {
                for (l=0; l<npix; l++) {
                    (*out)->data[m][k][l] = in[i]->data[j][k][l];
                }
                if (LINES  &&  !PROGRESS(psum+=pinc)) goto the_end;
            }
        }
    }

    the_end:
    if (TIMES) TIMING(T_EXIT);
}
