#ifdef HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */
#include        "sadie.h"

/*-Copyright Information------------------------------------------------------*/
/* Copyright (c) 1988 by the University of Arizona Digital Image Analysis Lab */
/*----------------------------------------------------------------------------*/
/*-General Information--------------------------------------------------------*/
/*                                                                            */
/*   This function generates a binary checkerboard pattern. The graylevel     */
/*   of the check in the upper left corner is SMIN, and the remaining         */
/*   checks alternate between SMAX and SMIN. Appropriate parameters will      */
/*   generate a binary bar pattern.                                           */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/*-Interface Information------------------------------------------------------*/
void CHECKER (
uint32_t nlin,  /*  I   Number of lines.                                      */
uint32_t npix,  /*  I   Number of pixels/line.                                */
uint32_t jsize, /*  I   Number of lines per check.                            */
uint32_t ksize, /*  I   Number of pixels/line per check.                      */
IMAGE **out     /*  O   Address of a pointer to the output image.             */
/*----------------------------------------------------------------------------*/
) { register uint32_t j, k;
    char msg[SLEN];

    if (TIMES) TIMING(T_CHECKER);
    if (NAMES) {
        MESSAGE('I',"");
        MESSAGE('I',"CHECKER");
        MESSAGE('I',"");
        sprintf(msg," Image size: lines:    %lu",nlin);
        MESSAGE('I',msg);
        sprintf(msg,"             pixels:   %lu",npix);
        MESSAGE('I',msg);
        sprintf(msg," Check size: lines:    %lu",jsize);
        MESSAGE('I',msg);
        sprintf(msg,"             pixels:   %lu",ksize);
        MESSAGE('I',msg);
        MESSAGE('I'," ...............");
    }

    /* check input */
    if (nlin <= 0) {
        MESSAGE('E'," Number of lines must be greater than zero.");
        goto the_end;
    } else if (npix <= 0) {
        MESSAGE('E'," Number of pixels/line must be greater than zero.");
        goto the_end;
    } else if (jsize <= 0) {
        MESSAGE('E'," Size of square in lines must be greater than zero.");
        goto the_end;
    } else if (jsize > nlin) {
        MESSAGE('E'," Size of square in lines must be less than or equal to the number of lines.");
        goto the_end;
    } else if (ksize <= 0) {
        MESSAGE('E'," Size of square in pixels/line must be greater than zero.");
        goto the_end;
    } else if (ksize > npix) {
        MESSAGE('E'," Size of square in pixels/line must be less than or equal to the number of pixels/line.");
        goto the_end;
    }

    /* create image of appropriate size */
    if (!CHECKIMG(*out)) GETMEM(1,nlin,npix,out);
    if (!*out) goto the_end;

    /* create checkerboard pattern */
    for (j=0; j<nlin; j++) {
        for (k=0; k<npix; k++) {
            (*out)->data[0][j][k] = (PIXEL)((j/jsize+k/ksize)%2 == 0  ?  SMIN : SMAX);
        }
    }
    (*out)->gmin = (PIXEL)SMIN;
    (*out)->gmax = (PIXEL)SMAX;
    
    the_end:
    if (TIMES) TIMING(T_EXIT);
}
