#ifdef HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */
#include        "sadie.h"

static PIXEL maskv[][3][3] =  { { { 1.0,  2.0,  1.0 }, { 0.0,  0.0,  0.0 }, {-1.0, -2.0, -1.0 } },
                                { { 1.0,  1.0,  1.0 }, { 0.0,  0.0,  0.0 }, {-1.0, -1.0, -1.0 } },
                                { { 1.0,  1.0,  1.0 }, {-1.0, -1.0, -1.0 }, { 0.0,  0.0,  0.0 } },
                                { { 0.0,  0.0,  0.0 }, { 0.0,  1.0,  0.0 }, { 0.0,  0.0, -1.0 } },
                                { { 0.0,  1.0,  0.0 }, { 0.0, -1.0,  0.0 }, { 0.0,  0.0,  0.0 } },
                                { { 1.0,  1.0,  1.0 }, { 1.0, -2.0,  1.0 }, {-1.0, -1.0, -1.0 } } };
static PIXEL maskh[][3][3] =  { { {-1.0,  0.0,  1.0 }, {-2.0,  0.0,  2.0 }, {-1.0,  0.0,  1.0 } },
                                { {-1.0,  0.0,  1.0 }, {-1.0,  0.0,  1.0 }, {-1.0,  0.0,  1.0 } },
                                { { 0.0, -1.0,  1.0 }, { 0.0, -1.0,  1.0 }, { 0.0, -1.0,  1.0 } },
                                { { 0.0,  0.0,  0.0 }, { 0.0,  0.0,  1.0 }, { 0.0, -1.0,  0.0 } },
                                { { 0.0,  0.0,  0.0 }, { 0.0, -1.0,  1.0 }, { 0.0,  0.0,  0.0 } },
                                { {-1.0,  1.0,  1.0 }, {-1.0, -2.0,  1.0 }, {-1.0,  1.0,  1.0 } } };

/*-Copyright Information------------------------------------------------------*/
/* Copyright (c) 1988 by the University of Arizona Digital Image Analysis Lab */
/*----------------------------------------------------------------------------*/
/*-General Information--------------------------------------------------------*/
/*                                                                            */
/*   This function computes a two-dimensional gradient (magnitude and         */
/*   direction) of an image, using standard operators.                        */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/*-Background Information-----------------------------------------------------*/
/*                                                                            */
/*   Robinson, G.S.:                                                          */
/*   "Detection and Coding of Edges Using Directional Masks."                 */
/*   Optical Engineering, Vol. 16, No. 6 (Nov/Dec 1977), pp. 590-585          */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/*-Interface Information------------------------------------------------------*/
void EDGE (
IMAGE *in,      /*  I   Pointer to the input image.                           */
short option,   /*  I   Switch, indicating the gradient operator:             */
                /*      SOBEL                                                 */
                /*      SOBEL2                                                */
                /*      SOBEL3                                                */
                /*      ROBERTS                                               */
                /*      ROBERTS2                                              */
                /*      PREWITT                                               */
IMAGE **out1,   /*  O   Address of a pointer to the output image              */
                /*      containing the gradient magnitude.                    */
IMAGE **out2    /*  O   Address of a pointer to the output image              */
                /*      containing the gradient direction.                    */
/*----------------------------------------------------------------------------*/
) { char msg[SLEN];
    
    if (TIMES) TIMING(T_EDGE);
    if (NAMES) {
        MESSAGE('I',"");
        MESSAGE('I',"EDGE");
    }

    /* check input */
    if (!CHECKIMG(in)) {
        MESSAGE('E'," Can't identify image.");
        goto the_end;
    } else if (option != SOBEL  &&  option != SOBEL2  &&  option != SOBEL3  &&  option != ROBERTS  &&  option != ROBERTS2  &&  option != PREWITT) {
        MESSAGE('E'," Option must be one of SOBEL, SOBEL2, SOBEL3, ROBERTS, ROBERTS2, or PREWITT.");
        goto the_end;
    }

    if (NAMES) {
        MESSAGE('I',"");
        sprintf(msg," Input image:   %s",in->text);
        MESSAGE('I',msg);
        if (option == SOBEL) {
            MESSAGE('I'," SOBEL gradient operator"); 
        } else if (option == SOBEL2) {
            MESSAGE('I'," SOBEL2 gradient operator"); 
        } else if (option == SOBEL3) {
            MESSAGE('I'," SOBEL3 gradient operator"); 
        } else if (option == ROBERTS) {
            MESSAGE('I'," ROBERTS gradient operator"); 
        } else if (option == ROBERTS2) {
            MESSAGE('I'," ROBERTS2 gradient operator"); 
        } else if (option == PREWITT) {
            MESSAGE('I'," PREWITT gradient operator"); 
        }
        MESSAGE('I'," ...............");
    }

    /* compute 2D-gradient of the image */
    GRADIENT(in,(PIXEL *)maskv[option],(PIXEL *)maskh[option],3,out1,out2);

    the_end:
    if (TIMES) TIMING(T_EXIT);
}
