#ifdef HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */
#include        "sadie.h"

/*-Copyright Information------------------------------------------------------*/
/* Copyright (c) 1988 by the University of Arizona Digital Image Analysis Lab */
/*----------------------------------------------------------------------------*/
/*-General Information--------------------------------------------------------*/
/*                                                                            */
/*   This function expands or contracts an image by scale factors.            */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/*-Interface Information------------------------------------------------------*/
void EXPAND (
IMAGE  *in,     /*  I   Pointer to the input image.                           */
double jfact,   /*  I   Line scale factor.                                    */
double kfact,   /*  I   Pixel scale factor.                                   */
double option,  /*  I   Switch, determines interpolation method:              */
                /*      < 0.0   -   Cubic interpolation (alpha = option).     */
                /*      = 0.0   -   Bilinear interpolation.                   */
                /*      > 0.0   -   Nearest neighbor interpolation.           */
IMAGE  **out    /*  O   Address of a pointer to the output image.             */
/*----------------------------------------------------------------------------*/
) { register short i;
    char   msg[SLEN];
    double xc[3], yc[3];

    if (TIMES) TIMING(T_EXPAND);
    if (NAMES) {
        MESSAGE('I',"");
        MESSAGE('I',"EXPAND");
    }

    /* check input */
    if (!CHECKIMG(in)) {
        MESSAGE('E'," Can't identify image.");
        goto the_end;
    } else if (jfact <= 0.0) {
        MESSAGE('E'," Line scaling factor must be greater than zero.");
        goto the_end;
    } else if (kfact <= 0.0) {
        MESSAGE('E'," Pixel scaling factor must be greater than zero.");
        goto the_end;
    }

    if (NAMES) {
        MESSAGE('I',"");
        sprintf(msg," Input image:              %s",in->text);
        MESSAGE('I',msg);
        sprintf(msg," Scale factor: lines:    %12.4e",jfact);
        MESSAGE('I',msg);
        sprintf(msg,"               pixels:   %12.4e",kfact);
        MESSAGE('I',msg);
        if (option < 0.0) {
            sprintf(msg," Parametric cubic interpolation, alpha = %5.2f",option);
            MESSAGE('I',msg);
        } else if (option == 0.0) {
            MESSAGE('I'," Bilinear interpolation");
        } else if (option > 0.0) {
            MESSAGE('I'," Nearest-neighbor interpolation");
        }
        MESSAGE('I'," ...............");
    }

    /* compute transformation coefficients and perform transformation */
    if (jfact == (double)((uint32_t)jfact)  &&  kfact == (double)((uint32_t)kfact)  &&  option > 0.0) {
        DUPL(in,0,0,0,1,(uint32_t)jfact,(uint32_t)kfact,in->nbnd,in->nlin,in->npix,out);
    } else {
        for (i=0; i<3; i++) {
        	xc[i] = 0.0;
        	yc[i] = 0.0;
        }
        xc[1] = 1.0/kfact;
        yc[2] = 1.0/jfact;
        GEOMWARP(in,in->nbnd,(uint32_t)ceil(jfact*(double)in->nlin),(uint32_t)ceil(kfact*(double)in->npix),xc,yc,3,option,(PIXEL)SMIN,out);
    }

    the_end:
    if (TIMES) TIMING(T_EXIT);
}
