#ifdef HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */
#include        "sadie.h"

/*-Copyright Information------------------------------------------------------*/
/* Copyright (c) 1988 by the University of Arizona Digital Image Analysis Lab */
/*----------------------------------------------------------------------------*/
/*-General Information--------------------------------------------------------*/
/*                                                                            */
/*   This function generates a horizontal grayscale pattern.                  */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/*-Interface Information------------------------------------------------------*/
void GRAYSCAL (
uint32_t nlin,  /*  I   Number of lines in the output image.                  */
uint32_t npix,  /*  I   Number of pixels/line in the output image.            */
uint32_t nlev,  /*  I   Number of graylevels in the output image.             */
PIXEL gmin,     /*  I   Minimum graylevel.                                    */
PIXEL gmax,     /*  I   Maximum graylevel.                                    */
IMAGE **out     /*  O   Address of a pointer to the output image.             */
/*----------------------------------------------------------------------------*/
) { register uint32_t j, k;
    register PIXEL p;
    char   msg[SLEN];
    double inc=(nlev<=1 ? 0.0:((double)(gmax-gmin)*(double)nlev)/((double)(nlev-1)*(double)npix));

    if (TIMES) TIMING(T_GRAYSCAL);
    if (NAMES) {
        MESSAGE('I',"");
        MESSAGE('I',"GRAYSCAL");
        MESSAGE('I',"");
        sprintf(msg," Image size: lines:     %lu",nlin);
        MESSAGE('I',msg);
        sprintf(msg,"             pixels:    %lu",npix);
        MESSAGE('I',msg);
        sprintf(msg," Minimum graylevel:   %12.4e",gmin);
        MESSAGE('I',msg);
        sprintf(msg," Maximum graylevel:   %12.4e",gmax);
        MESSAGE('I',msg);
        sprintf(msg," Number of graylevels:  %lu",nlev);
        MESSAGE('I',msg);
        MESSAGE('I'," ...............");
    }

    /* check input */
    if (nlin <= 0) {
        MESSAGE('E'," Number of lines must be greater than zero.");
        goto the_end;
    } else if (npix <= 0) {
        MESSAGE('E'," Number of pixels/line must be greater than zero.");
        goto the_end;
    } else if (nlev <= 0) {
        MESSAGE('E'," Number of graylevels must be greater than zero.");
        goto the_end;
    } else if (nlev > npix) {
        MESSAGE('E'," Number of graylevels must be less than or equal to number of pixels/line.");
        goto the_end;
    } else if (gmin > gmax) {
        MESSAGE('E', " Minimum graylevel value must be less than or equal to maximum graylevel value.");
        goto the_end;
    }

    /* create image of appropriate size */
    if (!CHECKIMG(*out)) GETMEM(1,nlin,npix,out);
    if (!*out) goto the_end;

    /* create grayscale */
    for (k=0; k<npix; k++) {
        p = (PIXEL)((double)gmin+(double)k*inc);
        for (j=0; j<nlin; j++) {
            (*out)->data[0][j][k] = p;
        }
    }
    (*out)->gmin = gmin;
    (*out)->gmax = gmax;

    the_end:
    if (TIMES) TIMING(T_EXIT);
}
