#ifdef HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */
#include "sadie.h"

#if defined SADIE_MACOS
#include <TextUtils.h>
#include <Files.h>
#endif

/* Copyright Information ---------------------------------------------------- */
/* Copyright (c) 1995 by the University of Arizona Digital Image Analysis Lab */
/* -------------------------------------------------------------------------- */
/* General Information ------------------------------------------------------ */
/*                                                                            */
/*    This function writes SADIE image data to an output file in ASCII        */
/*    format. Image header information is not written to the output file.     */
/*    The output file is band sequential, pixels are delimited by tabs,       */
/*    and the last pixel in each line is followed by a line return.           */
/*                                                                            */
/* -------------------------------------------------------------------------- */
/* Interface Information ---------------------------------------------------- */
void IMG2TEXT (
IMAGE *in,           /*  I  Pointer to the input image.                       */
unsigned char *name, /*  I  String containing the name of the ASCII file.     */
short ref,           /*  I  Reference number for the disk file.               */
short vref           /*  I  Volume reference number for the disk file.        */
/* -------------------------------------------------------------------------- */
) { register uint32_t i, j, k;
    char  msg[SLEN];
#if defined SADIE_UNIX
    FILE  *fp = NULL;
#elif defined SADIE_MACOS
    short error;
    long nbytes;
    Str255 pstr;
#endif

    if (TIMES) TIMING(T_IMG2TEXT);
    if (NAMES) {
        MESSAGE('I',"");
        MESSAGE('I',"IMG2TEXT");
    }

    /* check input */
    if (!CHECKIMG(in)) {
        MESSAGE('E'," Can't identify image.");
        goto the_end;
    }

    if (NAMES) {
        MESSAGE('I',"");
        sprintf(msg," Input image:               %s",in->text);
        MESSAGE('I',msg);
        sprintf(msg," Output disk file:          %s",name);
        MESSAGE('I',msg);
        sprintf(msg," Output format:             ASCII Text");
        MESSAGE('I',msg);
        MESSAGE('I'," ...............");
    }
 
#if defined SADIE_UNIX
    /* open image file */
    if (!(fp = fopen(name,"w"))) {
        sprintf(msg," Can't open file %s.",name);
        MESSAGE('E',msg);
        goto the_end;
    }
#endif

    /* write image data to file */
    for (i=0; i<in->nbnd; i++) {
        for (j=0; j<in->nlin; j++) {
#if defined SADIE_UNIX
            for (k=0; k<in->npix-1; k++) {
                if (fprintf(fp,"%.4f ",in->data[i][j][k]) < 1) {
                    sprintf(msg," Can't write image data to file %s.",name);
                    MESSAGE('E',msg);
                    goto the_end;
                }
            }
            if (fprintf(fp,"%.4f\n",in->data[i][j][k]) < 1) {
                sprintf(msg," Can't write image data to file %s.",name);
                MESSAGE('E',msg);
                goto the_end;
            }
#elif defined SADIE_MACOS
            for (k=0; k<in->npix-1; k++) {
                sprintf(msg,"%12.5E\t",in->data[i][j][k]);
                nbytes = strlen(msg);
                if ((error=FSWrite(ref,&nbytes,msg)) != noErr) {
                    sprintf(msg," Can't write image data to file %s.\rFile Manager Error Code: %d.",name,error);
                    MESSAGE('E',msg);
                    goto the_end;
                }
            }
            sprintf(msg,"%12.5E\r",in->data[i][j][k]);
            nbytes = strlen(msg);
            if ((error=FSWrite(ref,&nbytes,msg)) != noErr) {
                sprintf(msg," Can't write image data to file %s.\rFile Manager Error Code: %d.",name,error);
                MESSAGE('E',msg);
                goto the_end;
            }
#endif
        }
    }

#if defined SADIE_MACOS
    c2pstrcpy(pstr,(char *)name);
    if ((error=FlushVol(pstr,vref)) != noErr  ||  (error=GetFPos(ref,&nbytes)) != noErr  ||  (error=SetEOF(ref,nbytes)) != noErr) {
        sprintf(msg," Can't write image data to file %s.\rFile Manager Error Code: %d.",name,error);
        MESSAGE('E',msg);
        goto the_end;
    }
#endif

    /* close image file */
    the_end:
#if defined SADIE_UNIX
    if (fp) {
        if (fclose(fp)) {
            sprintf(msg," Can't close file %s.",name);
            MESSAGE('W',msg);
        }
    }
#elif defined SADIE_MACOS
    if ((error=FSClose(ref)) != noErr) {
        sprintf(msg," Can't close file %s.\rFile Manager Error Code: %d.",name,error);
        MESSAGE('W',msg);
    }
#endif
    if (TIMES) TIMING(T_EXIT);
}        
