#ifdef HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */
#include        "sadie.h"

/*-Copyright Information------------------------------------------------------*/
/* Copyright (c) 1993 by the University of Arizona Digital Image Analysis Lab */
/*----------------------------------------------------------------------------*/
/*-General Information--------------------------------------------------------*/
/*                                                                            */
/*   This function rotates an image.                                          */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/*-Interface Information------------------------------------------------------*/
void ROTATE (
IMAGE  *in,     /*  I   Pointer to the input image.                           */
double angle,   /*  I   Rotation angle,                                       */
                /*      measured in degrees clockwise from the horizon.       */
PIXEL  glev,    /*  I   Graylevel to fill otherwise empty pixels.             */
double option,  /*  I   Switch, determines interpolation method:              */
                /*      < 0.0   -   Cubic interpolation (alpha = option).     */
                /*      = 0.0   -   Bilinear interpolation.                   */
                /*      > 0.0   -   Nearest neighbor interpolation.           */
IMAGE  **out    /*  O   Address of a pointer to the output image.             */
/*----------------------------------------------------------------------------*/
) { register short i;
    char   msg[SLEN];
    double s, w, xmin, xmax, ymin, ymax, x[4], y[4], xp[4], yp[4], xc[3], yc[3];

    if (TIMES) TIMING(T_ROTATE);
    if (NAMES) {
        MESSAGE('I',"");
        MESSAGE('I',"ROTATE");
    }

    /* check input */
    if (!CHECKIMG(in)) {
        MESSAGE('E'," Can't identify image.");
        goto the_end;
    } else if (angle < -360.0  ||  360.0 < angle) {
        MESSAGE('E'," Rotation angle must be between -360 and +360 degrees.");
        goto the_end;
    }

    if (NAMES) {
        MESSAGE('I',"");
        sprintf(msg," Input image:                %s",in->text);
        MESSAGE('I',msg);
        sprintf(msg," Rotation:                   %12.4e",angle);
        MESSAGE('I',msg);
        sprintf(msg," Fill graylevel:             %12.4e",glev);
        MESSAGE('I',msg);
        if (option < 0.0) {
            sprintf(msg," Parametric cubic interpolation, alpha = %5.2f",option);
            MESSAGE('I',msg);
        } else if (option == 0.0) {
            MESSAGE('I'," Bilinear interpolation");
        } else if (option > 0.0) {
            MESSAGE('I'," Nearest-neighbor interpolation");
        }
        MESSAGE('I'," ...............");
    }

    if (angle == (double)((short)angle)  &&  ((short)angle)%90 == 0  &&  option > 0.0) {
        /* perform transformation directly */
        if ((short)angle == -360  ||  (short)angle == 0  ||  (short)angle == 360) {
            TRNSFORM(in,ROTPM0,out);
        } else if ((short)angle == -270  ||  (short)angle == 90) {
            TRNSFORM(in,ROTP90,out);
        } else if ((short)angle == -180  ||  (short)angle == 180) {
            TRNSFORM(in,ROT180,out);
        } else if ((short)angle == -90  ||  (short)angle == 270) {
            TRNSFORM(in,ROTM90,out);
        }
    } else {
        /* compute transformation coefficients and perform transformation */
        s = sqrt((double)in->npix*(double)in->npix + (double)in->nlin*(double)in->nlin) / 2.0;
        for (i=0; i<4; i++) {
            w = i*PI/2.0 + atan(i%2 ? (double)in->npix/(double)in->nlin : (double)in->nlin/(double)in->npix);
            x[i]  = (double)in->npix/2.0 + s*cos(w);
            y[i]  = (double)in->nlin/2.0 + s*sin(w);
            xp[i] = (double)in->npix/2.0 + s*cos(w+PI*angle/180.0);
            yp[i] = (double)in->nlin/2.0 + s*sin(w+PI*angle/180.0);
            if (i == 0) {
                xmin = xp[i];
                xmax = xp[i];
                ymin = yp[i];
                ymax = yp[i];
            } else {
                xmin = min(xmin,xp[i]);
                xmax = max(xmax,xp[i]);
                ymin = min(ymin,yp[i]);
                ymax = max(ymax,yp[i]);
            }
        }
        for (i=0; i<4; i++) {
            xp[i] -= xmin;
            yp[i] -= ymin;
        }
        GEOMCOEF(x,y,xp,yp,4,3,xc,yc);
        GEOMWARP(in,in->nbnd,(uint32_t)(ymax-ymin)+((ymax-ymin) > (double)(uint32_t)(ymax-ymin)),(uint32_t)(xmax-xmin)+((xmax-xmin) > (double)(uint32_t)(xmax-xmin)),xc,yc,3,option,glev,out);
    }

    the_end:
    if (TIMES) TIMING(T_EXIT);
}
