#ifdef HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */
#include        "sadie.h"

/*-Copyright Information------------------------------------------------------*/
/* Copyright (c) 1993 by the University of Arizona Digital Image Analysis Lab */
/*----------------------------------------------------------------------------*/
/*-General Information--------------------------------------------------------*/
/*                                                                            */
/*   This function creates a sine star test pattern.                          */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/*-Interface Information------------------------------------------------------*/
void SINESTAR (
uint32_t nlin,  /*  I   Number of lines in the output image.                  */
uint32_t npix,  /*  I   Number of pixels/line in the output image.            */
double period,  /*  I   Number of sine periods.                               */
IMAGE  **out    /*  O   Address of a pointer to the output image.             */
/*----------------------------------------------------------------------------*/
) { register uint32_t j, k;
    char   msg[SLEN];
    double x, y, angle;

    if (TIMES) TIMING(T_SINESTAR);
    if (NAMES) {
        MESSAGE('I',"");
        MESSAGE('I',"SINESTAR");
        MESSAGE('I',"");
        sprintf(msg," Image size: lines:           %lu",nlin);
        MESSAGE('I',msg);
        sprintf(msg,"             pixels:          %lu",npix);
        MESSAGE('I',msg);
        sprintf(msg," Period(s):                 %12.4e",period);
        MESSAGE('I',msg);
        MESSAGE('I'," ...............");
    }

    /* create image of appropriate size */
    if (!CHECKIMG(*out)) GETMEM(1,nlin,npix,out);

    /* create sine star test pattern */
    for (j=0; j<nlin; j++) {
        y = (double)j - (double)(nlin/2);
        for (k=0; k<npix; k++) {
            x = (double)k - (double)(npix/2);
            if (x == 0.0) {
                if (y > 0.0) {
                    angle =  PI / 2.0;
                } else if (y < 0.0) {
                    angle = -PI / 2.0;
                } else {
                    angle = 0.0;
                }
            } else {
                angle = atan2(y,x);
            }
            (*out)->data[0][j][k] = (PIXEL)cos(angle*period);
        }
    }

    the_end:
    if (TIMES) TIMING(T_EXIT);
}
