#ifdef HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */
#include "sadie.h"

#if defined SADIE_LIBTIFF

#include <tiffio.h>
#include <tiffiop.h>

jmp_buf  TIFFJmpBuf;

long     Warnings;

/*-Copyright Information------------------------------------------------------*/
/* Copyright (c) 2001 by the University of Arizona Digital Image Analysis Lab */
/*----------------------------------------------------------------------------*/
/*-General Information--------------------------------------------------------*/
/*                                                                            */
/*   This function reads a TIFF image from disk into main memory.             */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/*-Interface Information------------------------------------------------------*/
void TIFF2IMG (
unsigned char *name, /*  I   String, containing the name of the disk file.    */
short ref,           /*  I   Reference number of the disk file.               */
IMAGE **out          /*  O   Address of a pointer to the image.               */
/*----------------------------------------------------------------------------*/
) { register uint32_t j, k, *raster, *pixel;
    uint64_t total;
    char msg[SLEN];
    TIFF *tif;
    TIFFDirectory *td;
    static const char *photoNames[] = {"min-is-white","min-is-black","RGB color","palette color (RGB from colormap)","transparency mask","separated","YCbCr","7 (0x7)","CIE L*a*b*"};

    if (TIMES) TIMING(T_TIFF2IMG);

    if (setjmp(TIFFJmpBuf) == 0) {
        Warnings = 0;
#if defined SADIE_UNIX
        tif = TIFFOpen((const char *)name,"r");
#elif defined SADIE_MACOS
        tif = TIFFFdOpen((int)ref,(const char *)name,"r");
#endif
        TIFFReadDirectory(tif);
        td = &tif->tif_dir;
        total = (uint64_t)td->td_imagewidth * (uint64_t)td->td_imagelength * (uint64_t)sizeof(uint32_t);
        if (total > (uint64_t)SIZE_MAX  ||  (raster = (uint32_t *)malloc((size_t)total)) == NULL) {
            MESSAGE('E'," Memory request failed.");
            goto the_end;
        }
        TIFFReadRGBAImage(tif,td->td_imagewidth,td->td_imagelength,(uint32 *)raster,1);

        if (td->td_samplesperpixel == 1  &&  td->td_photometric != 3) {
            if (!CHECKIMG(*out)) GETMEM(1,td->td_imagelength,td->td_imagewidth,out);
            if (!*out) goto the_end;
            pixel = raster;
            for (j=(*out)->nlin-1; j>=0; j--) {
                for (k=0; k<(*out)->npix; k++) {
                    (*out)->data[0][j][k] = (PIXEL)TIFFGetR(*pixel);
                    pixel++;
                }
            }
        } else {
            if (!CHECKIMG(*out)) GETMEM(3,td->td_imagelength,td->td_imagewidth,out);
            if (!*out) goto the_end;
            pixel = raster;
            for (j=(*out)->nlin-1; j>=0; j--) {
                for (k=0; k<(*out)->npix; k++) {
                    (*out)->data[0][j][k] = (PIXEL)TIFFGetR(*pixel);
                    (*out)->data[1][j][k] = (PIXEL)TIFFGetG(*pixel);
                    (*out)->data[2][j][k] = (PIXEL)TIFFGetB(*pixel);
                    pixel++;
                }
            }
        }

        if (NAMES) {
            RANGE(*out);
            MESSAGE('I',"");
            MESSAGE('I',"TIFF2IMG");
            MESSAGE('I',"");
            sprintf(msg," Input file:                                 %s",name);
            MESSAGE('I',msg); 
            if (TIFFFieldSet(tif,FIELD_PHOTOMETRIC)) {
                if (td->td_photometric < sizeof(photoNames)/sizeof(photoNames[0])) {
                    sprintf(msg," Photometric Interpretation:                 %s",photoNames[td->td_photometric]);
                } else if (td->td_photometric == PHOTOMETRIC_LOGL) {
                    sprintf(msg," Photometric Interpretation:                 CIE Log2(L)");
                } else if (td->td_photometric == PHOTOMETRIC_LOGLUV) {
                    sprintf(msg," Photometric Interpretation:                 CIE Log2(L) (u',v')");
                } else {
                    sprintf(msg," Photometric Interpretation:                 %u (0x%x)",td->td_photometric,td->td_photometric);
                }
            }
            MESSAGE('I',msg); 
            sprintf(msg," Number of bands, lines, pixels:             %lu, %lu, %lu",(*out)->nbnd,(*out)->nlin,(*out)->npix);
            MESSAGE('I',msg);
            sprintf(msg," Number of bits/pixel/band (TIFF file):      %lu",(uint32_t)td->td_bitspersample);
            MESSAGE('I',msg);
            sprintf(msg," Graylevel minimum, maximum:                 %10.4e, %10.4e",(*out)->gmin,(*out)->gmax);
            MESSAGE('I',msg);
            MESSAGE('I'," ...............");
        }
        TIFFClose(tif);
    }

    the_end:
    if (raster) free(raster);
    if (TIMES)  TIMING(T_EXIT);
}
#endif
